__doc__ = """MyAdmin product module.
This module provides the methods and classes that Zope needs for the product. 
__init__.py is responsible for registering these methods and classes.
See README.txt for more details."""

__version__ = '0.1' 

from Globals import HTMLFile      
from Globals import MessageDialog 
from Globals import Persistent    

import OFS.SimpleItem
import Acquisition
import ts_regex
import AccessControl.Role

import regex, re, os, sys

import OFS.ObjectManager 
import OFS.CopySupport 
import OFS.Application

import Products.ZMySQLDA.DA
import Products.ZMySQLDA.db


_marker=[]
bad_id=ts_regex.compile('[^a-zA-Z0-9-_~\,\. ]').match 

tempVar=""
tmpConStr=""

"""Step #1: Provide "constructors" for Zope's management interface. 
appropriate DTML document based on a file sitting in your product 
directory. 
"""

manage_addMyAdminForm = HTMLFile('MyAdminAdd', globals()) 
def manage_addMyAdmin(self,id,title='',connection_string=None,REQUEST=None):
    """Add a MyAdmin to a folder."""
    self._setObject(id, MyAdmin(id, title, connection_string))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)


"""Step #2: Provide the class for your MyAdmin object."""
class MyAdmin(
    OFS.SimpleItem.Item,   
    Persistent,            
    Acquisition.Implicit,   
    AccessControl.Role.RoleManager,
    OFS.CopySupport.CopyContainer,
    OFS.ObjectManager.ObjectManager,
    OFS.Application.Application
    ): 
    """MyAdmin object. 
    
    This is an attempt at writing a product from scratch using mostly 
    Python instead of ZClasses. We'll see how it all goes.
    """
        
    meta_type = 'MyAdmin' 
    _objects   =()
    
    manage_options = ( 
	{'label': 'Contents',       'action': 'manage_main'},
	{'label': 'Properties',       'action': 'manage_editMyAdmin'},
	{'label': 'View',       'action': 'manage_view'}, 
        ) 

    __ac_permissions__=(
	('View management screens', ('manage_tabs','manage_main')),
	('Change MyAdmin'     ,     ('manage_editMyAdmin',)             ),
	('View MyAdmin',            ('manage_view',)                        ),
	)
	
    def __init__(self, id, title='', connection_string=None): 
        """initialize a new instance of MyAdmin"""
        global tmpConStr
        tmpConStr=connection_string
        
        self.id=id
        self.title=title
        self.connection_string=connection_string
        
        id1="MySQL_database_connection"
	title1="Z MySQL Database Connection"
	check="YES"
	
	self._setObject(id1, Connection(id1,title1,connection_string,check))
           
      
    manage_view = HTMLFile('index', globals()) 
    manage_editMyAdmin = HTMLFile('MyAdminEdit', globals()) 
    manage_main = HTMLFile('MyAdminImport', globals()) 
    
    def manage_importObject(self, REQUEST=None):
        """Import an object from a file"""
        file='zopeMyAdmin.zexp'
        file=os.path.join(INSTANCE_HOME, 'lib/python/Products/MyAdmin', file)
	ob=self._p_jar.importFile(file)
        if REQUEST:
            self._verifyObjectPaste(ob, REQUEST)
        id=ob.id
        if hasattr(id,'im_func'): id=id()
        self._setObject(id, ob)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(REQUEST['URL1']+'/zopeMyAdmin/manage_workspace')

    def _setObject(self,id,object,roles=None,user=None):
        v=self._checkId(id)
        if v is not None: id=v
        try:    t=object.meta_type
        except: t=None
        self._objects=self._objects+({'id':id,'meta_type':t},)
        self._setOb(id,object)
        object=self._getOb(id)
        object.manage_afterAdd(object, self)
        
        # Try to give user the local role "Owner", but only if
        # no local roles have been set on the object yet.
        if hasattr(self, 'REQUEST') and hasattr(object, '__ac_local_roles__'):
            if object.__ac_local_roles__ is None:
                user=self.REQUEST['AUTHENTICATED_USER']
                name=user.getUserName()
                if name != 'Anonymous User':
                    object.manage_setLocalRoles(name, ['Owner'])
        return id
        
    def _checkId(self, id, allow_dup=0):
        # If allow_dup is false, an error will be raised if an object
        # with the given id already exists. If allow_dup is true,
        # only check that the id string contains no illegal chars.
        if not id:
            raise 'Bad Request', 'No id was specified'
        if bad_id(id) != -1:
            raise 'Bad Request', (
            'The id %s contains characters illegal in URLs.' % id)
        if id[0]=='_': raise 'Bad Request', (
            'The id %s  is invalid - it begins with an underscore.'  % id)
        if not allow_dup:
            if hasattr(self, 'aq_base'):
                self=self.aq_base
            if hasattr(self, id):
                if id!="MySQL_database_connection":
                    raise 'Bad Request', (
                    'The id %s is invalid - it is already in use.' % id)
                else:
                    pass    
        if id == 'REQUEST':
            raise 'Bad Request', 'REQUEST is a reserved name.'
        if '/' in id:
            raise 'Bad Request', (
                'The id %s contains characters illegal in URLs.' % id
                )
        
    def manage_editFile(self, title, connection_string,  REQUEST=None): 
        """does this really need a doc string?"""
	global tempVar
        self.title = title
                
        id1="MySQL_database_connection"
	title1="Z MySQL Database Connection"
	check="YES"
	self.connection_string=connection_string
	self._setObject(id1, Connection(id1,title1,connection_string,check))
   
        if REQUEST is not None:
	    return MessageDialog(
		title = 'Edited',
		message = "Properties for %s changed." %self.id,
		action = './manage_main',
		)

    def _getOb(self, id, default=_marker):
        if hasattr(self, 'aq_base'):
            base=self.aq_base
        else: base=self
        if not hasattr(base, id):
            if default is _marker:
                raise AttributeError, id
            return default
        return getattr(self, id)
        
 
    def objectIds(self, spec=None):
        """Return a list of subobject ids.
        Returns a list of subobject ids of the current object.  If 'spec' is
        specified, returns objects whose meta_type matches 'spec'.
        """
        if spec is not None:
            if type(spec)==type('s'):
                spec=[spec]
            set=[]
            for ob in self._objects:
                if ob['meta_type'] in spec:
                    set.append(ob['id'])
            return set
        return map(lambda i: i['id'], self._objects)

    def objectItems(self, spec=None):
        """Return a list of (id, subobject) tuples.
        Returns a list of (id, subobject) tuples of the current object.
        If 'spec' is specified, returns only objects whose meta_type match
        'spec'
        """
        r=[]
        a=r.append
        g=self._getOb
        for id in self.objectIds(spec):
            if id !="MySQL_database_connection":
                a((id, g(id)))
            else:
                pass
        return r
       
class Connection(Products.ZMySQLDA.DA.Connection):
    """A database connection object"""
    pass
    
    def factory(self):
        return DB

class DB(Products.ZMySQLDA.db.DB):
    pass
	         
     
               

         
 	 

     			
     			
        
                         
                 
                 




   
    
    
        
